import 'package:auth_flutter/credential/google_credential.dart';
import 'package:auth_flutter/credential/login_credential.dart';
import 'package:auth_flutter/credential/phone_otp_credential.dart';
import 'package:auth_flutter/credential/username_password_credential.dart';
import 'package:auth_flutter/model/id_token.dart';
import 'package:auth_flutter/model/user_info.dart';
import 'package:auth_flutter/result/result.dart';
import 'package:auth_flutter/src/model_utils.dart';

import 'credential/apple_credential.dart';
import 'credential/custom_credential.dart';
import 'credential/facebook_credential.dart';
import 'model/iam_token.dart';
import 'src/auth_api.dart' as pigeon;

class TerraAuth {
  static final _flutterAuthApi = pigeon.FlutterAuthApi();
  static final Map<String, TerraAuth> _instances = {};

  final String _appName;

  TerraAuth._(this._appName);

  static Future<TerraAuth> getInstance(String appName) async {
    if (_instances[appName] == null) {
      _instances[appName] = await _createNewInstance(appName);
    }

    return _instances[appName]!;
  }

  static Future<TerraAuth> _createNewInstance(String appName) async {
    final newAuthInstance = TerraAuth._(appName);
    try {
      await _flutterAuthApi.getInstance(appName);
    } on Exception {
      rethrow;
    }

    return newAuthInstance;
  }

  Future<Result<bool, Exception>> isAuthorized() async {
    final result = await _flutterAuthApi.isAuthorized(_appName);

    if (result.isSuccess != true) {
      return Result.failure(Exception(result.error));
    }

    return Result.success(result.isAuthorized == true);
  }

  Future<Result<IamToken, Exception>> getAccessToken() async {
    final result = await _flutterAuthApi.getCurrentToken(_appName);

    if (result.isSuccess != true) {
      return Result.failure(Exception(result.error));
    }
    final pigeonIamToken = result.iamToken;

    if (pigeonIamToken == null) {
      return Result.failure(Exception("Missing data exception"));
    }

    final iamToken = ModelUtils.createTokenFromPigeon(pigeonIamToken);
    return Result.success(iamToken);
  }

  Future<Result<IamToken, Exception>> refreshToken() async {
    final result = await _flutterAuthApi.refreshToken(_appName);

    if (result.isSuccess != true) {
      return Result.failure(Exception(result.error));
    }
    final pigeonIamToken = result.iamToken;

    if (pigeonIamToken == null) {
      return Result.failure(Exception("Missing data exception"));
    }

    final iamToken = ModelUtils.createTokenFromPigeon(pigeonIamToken);
    return Result.success(iamToken);
  }

  Future<Result<IdToken, Exception>> exchangeToken(String iamAudience) async {
    final result = await _flutterAuthApi.exchangeToken(_appName, iamAudience);

    if (result.isSuccess != true) {
      return Result.failure(Exception(result.error));
    }
    final pigeonIdToken = result.idToken;

    if (pigeonIdToken == null) {
      return Result.failure(Exception("Missing data exception"));
    }

    final idToken = ModelUtils.createIdTokenFromPigeon(pigeonIdToken);
    return Result.success(idToken);
  }

  Future<Result<Object, Exception>> requestOtp(String phoneNumber) async {
    final result = await _flutterAuthApi.requestOtp(_appName, phoneNumber);

    if (result.isSuccess != true) {
      return Result.failure(Exception(result.error));
    }

    return Result.success(Object);
  }

  Future<Result<bool, Exception>> loginWithCredential(LoginCredential credential) async {
    if (credential is GoogleCredential) {
      return getLoginResult(() async {
        return await _flutterAuthApi.loginWithGoogleCredential(
            _appName, credential.toPigeonCredential());
      });
    }

    if (credential is FacebookCredential) {
      return getLoginResult(() async {
        return await _flutterAuthApi.loginWithFacebookCredential(
            _appName, credential.toPigeonCredential());
      });
    }

    if (credential is AppleCredential) {
      return getLoginResult(() async {
        return await _flutterAuthApi.loginWithAppleCredential(
            _appName, credential.toPigeonCredential());
      });
    }

    if (credential is UsernamePasswordCredential) {
      return getLoginResult(() async {
        return await _flutterAuthApi.loginWithUsernamePasswordCredential(
            _appName, credential.toPigeonCredential());
      });
    }

    if (credential is CustomCredential) {
      return getLoginResult(() async {
        return await _flutterAuthApi.loginWithCustomCredential(
            _appName, credential.toPigeonCredential());
      });
    }

    if (credential is PhoneOtpCredential) {
      return getLoginResult(() async {
        return await _flutterAuthApi.loginWithPhoneOtpCredential(
            _appName, credential.toPigeonCredential());
      });
    }

    return Result.failure(Exception("Unknown Credetial Exception"));
  }

  Future<Result<bool, Exception>> loginWithGoogle() async {
    return getLoginResult(() async {
      return await _flutterAuthApi.loginWithGoogle(_appName);
    });
  }

  Future<Result<bool, Exception>> loginWithApple() async {
    return getLoginResult(() async {
      return await _flutterAuthApi.loginWithApple(_appName);
    });
  }

  Future<Result<bool, Exception>> loginWithFacebook() async {
    return getLoginResult(() async {
      return await _flutterAuthApi.loginWithFacebook(_appName);
    });
  }

  Future<Result<bool, Exception>> logout() async {
    final result = await _flutterAuthApi.logout(_appName);

    if (result.isSuccess != true) {
      return Result.failure(Exception(result.error));
    }

    return Result.success(result.isLoggedOut == true);
  }

  Future<Result<bool, Exception>> getLoginResult(
      Future<pigeon.CommonAuthResult> Function() function) async {
    try {
      final result = await function.call();
      if (result.isSuccess != true) {
        return Result.failure(Exception(result.error));
      }

      return Result.success(true);
    } on Exception catch (error) {
      return Result.failure(error);
    }
  }

  Future<Result<UserInfo, Exception>> getUserInfo() async {
    final result = await _flutterAuthApi.getUserInfo(_appName);

    if (result.isSuccess != true) {
      return Result.failure(Exception(result.error ?? "Failed to get user info"));
    }

    if (result.userInfo == null) {
      return Result.failure(Exception("Missing data exception"));
    }

    return Result.success(ModelUtils.createUserInfoFromPigeon(result.userInfo!));
  }

  Future<Result<void, Exception>> verifyPhone(String phone) async {
    final result = await _flutterAuthApi.verifyPhone(_appName, phone);

    if (result.isSuccess != true) {
      return Result.failure(Exception(result.error));
    }
    return Result.success({});
  }

  Future<Result<void, Exception>> verifyEmail(String email) async {
    final result = await _flutterAuthApi.verifyEmail(_appName, email);

    if (result.isSuccess != true) {
      return Result.failure(Exception(result.error));
    }
    return Result.success({});
  }

  Future<Result<void, Exception>> confirmPhone(String phone, String code) async {
    final result = await _flutterAuthApi.confirmPhone(_appName, phone, code);

    if (result.isSuccess != true) {
      return Result.failure(Exception(result.error));
    }
    return Result.success({});
  }

  Future<Result<void, Exception>> confirmEmail(String email, String code) async {
    final result = await _flutterAuthApi.confirmEmail(_appName, email, code);

    if (result.isSuccess != true) {
      return Result.failure(Exception(result.error));
    }
    return Result.success({});
  }
}
